#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <stdio.h>

#include "init.h"
#include "support.h"
#include "judge.h"

//====================================================================================================
//メイン
//====================================================================================================

int main(int argc, char *argv[])
{
	//====================================================================================================
	//宣言
	//====================================================================================================
	unsigned char gflg=0;	//終了フラグ ・・・後々、修正する時に消す
	SDL_Surface *screen;	//スクリーン用メモリ
	SDL_Event event;	//イベント取得構造体

	//FPSに関する
	unsigned int s_time,e_time;
	float h_time;
	unsigned char s_flg=0;

	//画像に関する
	SDL_Surface *image1;	//画像読み込み用フラグ
	SDL_Surface *image2;	//画像読み込み用フラグ

	//描画に関する
	SDL_Rect src;			//転送元情報
	SDL_Rect dsr;			//転送先情報

	Mix_Music *title = NULL;		//BGM格納用
	Mix_Music *game = NULL;			//BGM格納用
	Mix_Chunk *cursor = NULL;		//効果音格納
	Mix_Chunk *select = NULL;		//効果音格納
	Mix_Chunk *bom = NULL;			//効果音格納
	int channel;				//効果音用チャンネル

	//ゲームに関する
	static int gameflg; //0 = タイトル 1 = ゲーム 2=リザルト

	//スコアに関する
	static int highscore, score;
	int scorecnt;

	//タイトルに関する
	static int startflg, endflg, titleflg; //ボタン描画切り替え用

	//マウスに関する
	static int mx, my;			//カーソル座標
	static int mbx, mby;			//クリックした座標

	//自機に関する
	int flg, jflg;				//状態フラグ,ジャンプフラグ
	int wx, wy, width, height;		//画像座標x,y,幅,高さ
	int x, y;				//座標x,y
	float vx, vy;				//ベクトルx,y
	float weight, gravity;			//体重,重力

	//弾に関する
	int tflg;				//状態フラグ
	int twx, twy, twidth, theight;		//画像座標x,y,幅,高さ
	int tx, ty;				//座標x,y
	float tvx, tvy;				//ベクトルx,y
	int tang;				//角度

	//ブロックに関する
	int blockwidth, blockheight;		//画像座標x,y,幅,高さ
	BLOCK block[BLOCKMAX];			//ブロックの構造体

	//ボールに関する
	int ballwidth, ballheight;		//画像座標x,y,幅,高さ
	BALL ball[BLOCKMAX];			//ボールの構造体

	//色に関する
	int white;				//塗りつぶし色格納用
	SDL_PixelFormat *fmt;			//サーフェスの色フォーマット格納用（パレット）

	//カウント用
	float v;				//ベクトル用
	int i;					//カウント
	static int cnt, bcnt;			//摩擦用,ブロック&ボール要素数用
	static int binitcnt;			//ブロック&ボール出現用
	static int keydraw;				//タイトル用(説明点滅)

	//====================================================================================================
	//初期化
	//====================================================================================================

	//ビデオシステムの初期化
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		return -1;
	}
	//タイトルバーのicon読み込み設定
	SDL_WM_SetIcon(SDL_LoadBMP("icon.bmp"),NULL);
	//タイトルバー名の設定
	SDL_WM_SetCaption("Closed Room (Ver 1.00)",NULL);

	//windowのビデオモード設定
	screen = SDL_SetVideoMode(VIDEO_W, VIDEO_H, VIDEO_BIT, SDL_SWSURFACE);
	if(screen == NULL){
		return -1;
	}

	srand(time(0));

	//画像読み込み
	image1 = (SDL_Surface *)IMG_Load("char.bmp");
	if(image1 == NULL) printf("Not Read File\n");
	image2 = (SDL_Surface *)IMG_Load("str.bmp");
	if(image2 == NULL) printf("Not Read File\n");

	//サウンド読み込み
	if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,2,1024) > 0){
		return(-1);
	}
	title = Mix_LoadMUS("title.wav");			//BGM用WAVファイル読み込み
	game = Mix_LoadMUS("game.wav");			//BGM用WAVファイル読み込み
//	result = Mix_LoadMUS("result.wav");		//BGM用WAVファイル読み込み
	cursor = Mix_LoadWAV("cursor.wav");		//効果音用WAVファイル読み込み
	select = Mix_LoadWAV("select.wav");		//効果音用WAVファイル読み込み
	bom = Mix_LoadWAV("bom.wav");			//効果音用WAVファイル読み込み

	//自機に関する
	//	MineInit(flg, jflg, wx, wy, width, height, x, y, vx, vy, weight, gravity);
	//弾に関する
	//	TamaInit(tflg, twx, twy, twidth, theight, tx, ty, tvx, tvy, tang);
	//ブロックに関する
	//	BlockInit(block, blockwidth, blockheight);
	//ボールに関する
	//	BallInit(block, ballwidth, ballheight);


	flg = 0;
	jflg = 0;
	wx = 0; wy = 0;
	width = 16; height = 32;
	x = VIDEO_W/2; y = VIDEO_H/2;
	vx = 0;	vy = 0;
	weight = 0.5; gravity = 0;

	tflg = 0;
	twx = 0, twy = 0, twidth = 0, theight = 0;
	tx = 0, ty = 0;
	tvx = 0, tvy = 0;
	tang = 0;

	blockwidth = 32, blockheight = 32;
	for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
		block[bcnt].wx = 64;			//画像座標x
		block[bcnt].wy = 0;			//画像座標y
		block[bcnt].x = -blockwidth;		//座標x
		block[bcnt].y = -blockheight;		//座標y
		block[bcnt].vx = 0;			//ベクトルx
		block[bcnt].vy = 0;			//ベクトルy
		block[bcnt].weight = 0.01;		//体重
		block[bcnt].gravity = 0;		//重力
		block[bcnt].flg = 0;			//状態
	}

	ballwidth = 32, ballheight = 32;
	for(bcnt = 0;bcnt < BALLMAX;bcnt++){
		ball[bcnt].wx = 96;			//画像座標x
		ball[bcnt].wy = 0;			//画像座標y
		ball[bcnt].x = -ballwidth;		//座標x
		ball[bcnt].y = -ballheight;		//座標y
		ball[bcnt].vx = 0;			//ベクトルx
		ball[bcnt].vy = 0;			//ベクトルy
		ball[bcnt].weight = 0.005;		//体重
		ball[bcnt].gravity = 0;			//重力
		ball[bcnt].flg = 0;			//状態
	}

	//透明色設定
	SDL_SetColorKey(image1,SDL_SRCCOLORKEY,0x00ff00ff); //紫

	unsigned char key_flg = 0x00;

	//BGM再生
	Mix_PlayMusic(title,-1);


	//====================================================================================================
	//ゲームループ
	//====================================================================================================
	while(gflg == 0){
		//キー操作==========================================
		while(SDL_PollEvent(&event)){ //イベント取得
			switch(event.type){

				case SDL_KEYDOWN:	//キー押された
					switch(event.key.keysym.sym){
				case SDLK_w: key_flg = key_flg | 0x01; break;
				case SDLK_s: key_flg = key_flg | 0x02; break;
				case SDLK_a: key_flg = key_flg | 0x04; break;
				case SDLK_d: key_flg = key_flg | 0x08; break;
				case ESC_KEY: gflg = 1; break;
					}
					break;
				case SDL_KEYUP:	//キー離された
					switch(event.key.keysym.sym){
				case SDLK_w: key_flg = key_flg & 0xfe; break;
				case SDLK_s: key_flg = key_flg & 0xfd; break;
				case SDLK_a: key_flg = key_flg & 0xfb; break;
				case SDLK_d: key_flg = key_flg & 0xf7; break;
					}
					break;
				case SDL_MOUSEMOTION:	//マウスが動いた
					if(gameflg == 0 || gameflg == 2){
						//マウスの座標をスプライト座標に設定
						mx = event.motion.x;
						my = event.motion.y;
					}
					break;

				case SDL_MOUSEBUTTONDOWN://マウスがクリックされた	
					switch(event.button.button){
				case SDL_BUTTON_LEFT:
					if(gameflg == 0 || gameflg == 2){
						tflg = 1;
						mbx = event.motion.x;
						mby = event.motion.y;
					}else if(gameflg == 1){
						if(tflg == 0){
							tflg = 1;
							//マウスの座標をスプライト座標に設定
							mbx = event.motion.x;
							mby = event.motion.y;
						}
					}
					break;

				case SDL_BUTTON_RIGHT:
					if(gameflg == 0){
						if(!keydraw) keydraw = 1;
						else keydraw = 0;
					}
					break;

				default:
					break;
					}
					break;

				case SDL_QUIT:		//プログラム終了
					gflg = 1;
					break;
			} //case END

		} //switch END

//====================================================================================================
//タイトル画面
//====================================================================================================
		if(gameflg == 0){
			if(flg == 0){ //自機生存時
				//操作処理==========================================
				if(key_flg & 0x01){
					if(jflg == 0){ vy = -10; jflg = 1;}
				}else{ if(jflg == 2) jflg = 0;}
				if(key_flg & 0x04){ vx-=1;}
				if(key_flg & 0x08){ vx+=1;}

				//==================================================
				//自機に関する
				//==================================================

				//付加==============================================
				if( CheckDownCollor(screen,x,y,width,height) == 0){ //下の色判定無
					//体重分を重力へ
					gravity += weight;

					//重力付加を自機へ
					vy += gravity;

					//体重分の重力をベクトルへ
					//vy += Gravity(weight, gravity);
				}

				//摩擦抵抗
				//Friction(cnt, vx, weight);

				//自機への抵抗付加
				if(vx < 0){ //左ベクトル時
					cnt++;
					if(cnt > weight*2){
						if(vx < -5) vx += 2.0;
						else vx+=1.0;
						if(vx > 0) vx = 0;
						cnt = 0;
					}
				}else if(vx > 0){ //右ベクトル時
					cnt++;
					if(cnt > weight*2){
						if(vx > 5) vx -= 2.0;
						else vx-=1.0;
						if(vx < 0) vx = 0;
						cnt = 0;
					}
				}else{
					cnt = 0;
				}

				//座標更新==========================================
				//下ベクトル
				if(vy > 0){
					for(v = 0;v < vy;v++){
						if( CheckDownCollor(screen,x,y,width,height) == 0){ //下の色判定無
							y++;
						}else{ //下の色判定有
							vy = 0;
							gravity = 0;
							jflg = 2;
							break;
						}
					}
				}
				//上ベクトル
				else if(vy < 0){
					for(v = 0;v > vy;v--){
						y--;
					}
				}
				//右ベクトル
				if(vx > 0){
					wy = 32;
					wx+=width;
					if(wx > width*3) wx = 0;
					for(v = 0;v < vx;v++){
						if(x+width <= 320){ //右の端判定無
							x++;
						}else{ //右の色判定有
							vx = 0;
							break;
						}
					}
				}
				//左ベクトル
				else if(vx < 0){
					wy = 64;
					wx+=width;
					if(wx > width*3) wx = 0;
					for(v = 0;v > vx;v--){
						if(x > 0){ //左の端判定無
							x--;
						}else{ //左の色判定有
							vx = 0;
							break;
						}
					}
				}
				//xベクトル無
				else{
					wy = 0;
					wx = 0;
				}
			}else{ //死亡時
				flg++; //2~
				//				wy += height;
				height -= 4;

				if(flg == 9){
					//自機に関する
//					MineInit(flg, jflg, wx, wy, width, height, x, y, vx, vy, weight, gravity);
					flg = 0;
					jflg = 0;
					wx = 0; wy = 0;
					width = 16; height = 32;
					x = VIDEO_W/2; y = VIDEO_H/2;
					vx = 0;	vy = 0;
					weight = 0.5; gravity = 0;
				}
			} //flg END

			//==================================================
			//ブロック処理
			//==================================================
			if( block[0].flg == 0){
				block[0].x = blockwidth;
				block[0].y = -blockheight/2;
				block[0].vy = 0;
				block[0].gravity = 0;
				block[0].flg = 1;

			}else if(block[0].flg == 1){
				block[0].gravity += block[0].weight;
				block[0].vy += block[0].gravity;
				if(block[0].vy > 0){
					for(v = 0;v < block[0].vy;v++){
						//自機潰し判定有
						if( CheckObjectDown(x,y,width,height,block[0].x,block[0].y,blockwidth,blockheight) == 1){
							flg = 1;
							wy = height*3;
							wx = 0;
							block[0].flg = 0;
							block[0].x = -blockwidth*2;
							break;
						}
						if(block[0].y <= 240){
							block[0].y++;
						}else{
							block[0].flg = 0;
							break;
						}
					}
				}
				if( CheckShot(mbx,mby,block[0].x,block[0].y,blockwidth,blockheight) == 1 && tflg == 1 ){ //判定有
					//効果音再生
					channel = Mix_PlayChannel(-1,bom,0);
					block[0].flg = 2;
				}

			}else if(block[0].flg == 2){ //破壊されたブロック
				block[0].wy+=blockheight; //画像の座標をずらしていく
				if(block[0].wy == blockheight*9){ //完全に消えたら
					block[0].flg = 0; //未出現にする
					block[0].wy = 0;
					block[0].x = -blockwidth;
				}
			} //block_flg END

			//==================================================
			//ボール処理
			//==================================================
			if( ball[0].flg == 0){
				ball[0].x = VIDEO_W-ballwidth*2;
				ball[0].y = -ballheight/2;
				ball[0].vy = 0;
				ball[0].gravity = 0;
				ball[0].flg = 1;

			}else if(ball[0].flg == 1){
				ball[0].gravity += ball[0].weight;
				ball[0].vy += ball[0].gravity;
				if(ball[0].vy > 0){
					for(v = 0;v < ball[0].vy;v++){
						//自機潰し判定有
						if( CheckObjectDown(x,y,width,height,ball[0].x,ball[0].y,ballwidth,ballheight) == 1){
							flg = 1;
							wy = height*3;
							wx = 0;
							ball[0].flg = 0;
							ball[0].x = -ballwidth*2;
							break;
						}
						if(ball[0].y <= 240){
							ball[0].y++;
						}else{
							ball[0].flg = 0;
							break;
						}
					}
				}
				if( CheckShot(mbx,mby,ball[0].x,ball[0].y,ballwidth,ballheight) == 1 && tflg == 1 ){ //判定有
					//効果音再生
					channel = Mix_PlayChannel(-1,bom,0);
					ball[0].flg = 2;
				}

			}else if(ball[0].flg == 2){ //破壊されたブロック
				ball[0].wy+=ballheight; //画像の座標をずらしていく
				if(ball[0].wy == ballheight*9){ //完全に消えたら
					ball[0].flg = 0; //未出現にする
					ball[0].wy = 0;
					ball[0].x = -ballwidth;
				}
			} //ball_flg END


			//すたーと カーソル
			if(VIDEO_W/2-50 < mx && VIDEO_W/2+50 > mx &&
				VIDEO_H/2 < my && VIDEO_H/2+30 > my){
					if(startflg == 0){
						//効果音再生
						channel = Mix_PlayChannel(-1,cursor,0);
						startflg = 1;
					}
			}else{
				startflg = 0;
			}
			//おわり カーソル
			if(VIDEO_W/2-50 < mx && VIDEO_W/2+50 > mx &&
				VIDEO_H/2+30 < my && VIDEO_H/2+60 > my){
					if(endflg == 0){
						//効果音再生
						channel = Mix_PlayChannel(-1,cursor,0);
						endflg = 1;
					}
			}else{
				endflg = 0;
			}

			if(tflg == 1){
				//すたーと クリック
				//				if( CheckShot(mbx,mby,VIDEO_W/2-50,VIDEO_H/2,100,30) == 1 ){
				if(VIDEO_W/2-50 < mbx && VIDEO_W/2+50 > mbx &&
					VIDEO_H/2 < mby && VIDEO_H/2+30 > mby){
						//効果音再生
						channel = Mix_PlayChannel(-1,select,0);
						gameflg = 1; //ゲーム開始
						//各初期化
/*
						//自機に関する
						MineInit(flg, jflg, wx, wy, width, height, x, y, vx, vy, weight, gravity);
						//弾に関する
						TamaInit(tflg, twx, twy, twidth, theight, tx, ty, tvx, tvy, tang);
						//ブロックに関する
						BlockInit(block, blockwidth, blockheight);
						//ボールに関する
						BallInit(block, ballwidth, ballheight);
*/
						score = 0;
						binitcnt = 0;

						flg = 0;
						jflg = 0;
						wx = 0; wy = 0;
						width = 16; height = 32;
						x = VIDEO_W/2; y = VIDEO_H/2;
						vx = 0;	vy = 0;
						weight = 0.5; gravity = 0;

						tflg = 0;
						twx = 0, twy = 0, twidth = 0, theight = 0;
						tx = 0, ty = 0;
						tvx = 0, tvy = 0;
						tang = 0;

						blockwidth = 32, blockheight = 32;
						for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
							block[bcnt].wx = 64;			//画像座標x
							block[bcnt].wy = 0;			//画像座標y
							block[bcnt].x = -blockwidth;		//座標x
							block[bcnt].y = -blockheight;		//座標y
							block[bcnt].vx = 0;			//ベクトルx
							block[bcnt].vy = 0;			//ベクトルy
							block[bcnt].weight = 0.01;		//体重
							block[bcnt].gravity = 0;		//重力
							block[bcnt].flg = 0;			//状態
						}

						ballwidth = 32, ballheight = 32;
						for(bcnt = 0;bcnt < BALLMAX;bcnt++){
							ball[bcnt].wx = 96;			//画像座標x
							ball[bcnt].wy = 0;			//画像座標y
							ball[bcnt].x = -ballwidth;		//座標x
							ball[bcnt].y = -ballheight;		//座標y
							ball[bcnt].vx = 0;			//ベクトルx
							ball[bcnt].vy = 0;			//ベクトルy
							ball[bcnt].weight = 0.005;		//体重
							ball[bcnt].gravity = 0;			//重力
							ball[bcnt].flg = 0;			//状態
						}	

						Mix_HaltMusic(); //BGM停止
						Mix_PlayMusic(game,-1); //BGM再生
				}
				//おわり クリック
				//				if( CheckShot(mbx,mby,VIDEO_W/2-50,VIDEO_H/2+30,100,30) == 1 ){
				if(VIDEO_W/2-50 < mbx && VIDEO_W/2+50 > mbx &&
					VIDEO_H/2+30 < mby && VIDEO_H/2+60 > mby){
						//効果音再生
						channel = Mix_PlayChannel(-1,select,0);
						gflg = 1; //ゲーム終了
				}
				tflg = 0;
			}

			//==================================================
			//描画処理
			//==================================================

			fmt = screen->format;						//サーフェスからパレット情報格納
			white = SDL_MapRGB(fmt, 0xff, 0xff, 0xff);	//R,G,B をパレット情報に合わせて格納
			//指定した色で塗りつぶす
			SDL_FillRect(screen, NULL, white);			//格納した色で塗りつぶす

			//自機描画
			src.w = width; src.h = height;
			src.x = wx; src.y = wy;
			dsr.x = x; dsr.y = y;
			SDL_BlitSurface(image1,&src,screen,&dsr);

			//ブロック描画
			src.w = blockwidth; src.h = blockheight;
			src.x = block[0].wx; src.y = block[0].wy;
			dsr.x = block[0].x; dsr.y = block[0].y;
			SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			//ボール描画
			src.w = ballwidth; src.h = ballheight;
			src.x = ball[0].wx; src.y = ball[0].wy;
			dsr.x = ball[0].x; dsr.y = ball[0].y;
			SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			//タイトル
			src.w = 160; src.h = 30;
			src.x = 0; src.y = 162;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = 60;
			SDL_BlitSurface(image2,&src,screen,&dsr);

			//すたーと
			src.w = 100; src.h = 30;
			src.x = 100*startflg; src.y = 20;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = VIDEO_H/2;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			//おわり
			src.x = 100*endflg; src.y = 50;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = VIDEO_H/2+30;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			//HIGHSCORE
			src.w = 80; src.h = 20;
			src.x = 50; src.y = 110;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = 0;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			for(i = 0;i < 3;i++){
				switch(i){
					case 0: scorecnt = highscore/100; break;
					case 1: scorecnt = (highscore/10)%10; break;
					case 2: scorecnt = highscore%10; break;
				}
				src.w = 15; src.h = 20;
				src.x = src.w*scorecnt; src.y = 0;
				dsr.x = (VIDEO_W/2-src.w-src.w/2)+src.w*i; dsr.y = 20;
				SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//説明
			if(keydraw == 0){
				src.w = 16; src.h = 16;

				//左
				src.x = 0; src.y = 130;
				dsr.x = x-src.w; dsr.y = y+src.h/2;
				SDL_BlitSurface(image2,&src,screen,&dsr);
				//上
				src.x = 16; src.y = 130;
				dsr.x = x; dsr.y = y-src.h;
				SDL_BlitSurface(image2,&src,screen,&dsr);
				//右
				src.x = 32; src.y = 130;
				dsr.x = x+src.w; dsr.y = y+src.h/2;
				SDL_BlitSurface(image2,&src,screen,&dsr);

				//左クリック
				src.w = 56; src.h = 16;
				src.x = 48; src.y = 130;
				//ブロック下に描画
				dsr.x = block[0].x+blockwidth/2-src.w/2; dsr.y = block[0].y+blockheight;
				SDL_BlitSurface(image2,&src,screen,&dsr);
				//ボール下に描画
				src.x = 48; src.y = 130;
				dsr.x = ball[0].x+ballwidth/2-src.w/2; dsr.y = ball[0].y+ballheight;
				SDL_BlitSurface(image2,&src,screen,&dsr);

				//右クリック ON/OFF
				src.w = 128; src.h = 16;
				src.x = 0; src.y = 146;
				dsr.x = VIDEO_W-src.w; dsr.y = VIDEO_H-src.h;
				SDL_BlitSurface(image2,&src,screen,&dsr);
			}

//====================================================================================================
//ゲーム画面
//====================================================================================================
		}else if(gameflg == 1){

			if(flg == 0){ //自機生存時
				//操作処理==========================================
				if(key_flg & 0x01){
					if(jflg == 0){ vy = -10; jflg = 1;}
				}else{ if(jflg == 2) jflg = 0;}
				//if(key_flg & 0x02){ score++;}
				if(key_flg & 0x04){ vx-=1;}
				if(key_flg & 0x08){ vx+=1;}

				//==================================================
				//自機に関する
				//==================================================

				//付加==============================================
				if( CheckDownCollor(screen,x,y,width,height) == 0){ //下の色判定無
					//体重分を重力へ
					gravity += weight;

					//重力付加を自機へ
					vy += gravity;

					//体重分の重力をベクトルへ
					//vy += Gravity(weight, gravity);
				}

				//摩擦抵抗
				//Friction(cnt, vx, weight);

				//自機への抵抗付加
				if(vx < 0){ //左ベクトル時
					cnt++;
					if(cnt > weight*2){
						if(vx < -5) vx += 2.0;
						else vx+=1.0;
						if(vx > 0) vx = 0;
						cnt = 0;
					}
				}else if(vx > 0){ //右ベクトル時
					cnt++;
					if(cnt > weight*2){
						if(vx > 5) vx -= 2.0;
						else vx-=1.0;
						if(vx < 0) vx = 0;
						cnt = 0;
					}
				}else{
					cnt = 0;
				}

				//座標更新==========================================
				//下ベクトル
				if(vy > 0){
					for(v = 0;v < vy;v++){
						if( CheckDownCollor(screen,x,y,width,height) == 0){ //下の色判定無
							y++;
						}else{ //下の色判定有
							vy = 0;
							gravity = 0;
							jflg = 2;
							break;
						}
					}
				}
				//上ベクトル
				else if(vy < 0){
					for(v = 0;v > vy;v--){
						if( CheckUpCollor(screen,x,y,width,height) == 0){ //上の色判定無
							y--;
							if( CheckUp(screen,x,y,width,height) == 1){ //天井の判定有
								flg = 1;
							}
						}else{ //上の色判定有
							vy = 0;
							break;
						}
					}
				}
				//右ベクトル
				if(vx > 0){	
					wy = 32;
					wx+=width;
					if(wx > width*3) wx = 0;
					for(v = 0;v < vx;v++){
						if( CheckRightCollor(screen,x,y,width,height) == 0){ //右の色判定無
							x++;
						}else{ //右の色判定有
							vx = 0;
							break;
						}
					}
				}
				//左ベクトル
				else if(vx < 0){
					wy = 64;
					wx+=width;
					if(wx > width*3) wx = 0;
					for(v = 0;v > vx;v--){
						if( CheckLeftCollor(screen,x,y,width,height) == 0){ //左の色判定無
							x--;
						}else{ //左の色判定有
							vx = 0;
							break;
						}
					}
				}
				//xベクトル無
				else{
					wy = 0;
					wx = 0;
				}

				//==================================================
				//弾に関する
				//==================================================
				if(tflg == 1){
					//				tx += tvx;
					//				ty += tvy;
					for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
						if( CheckShot(mbx,mby,block[bcnt].x,block[bcnt].y,blockwidth,blockheight) == 1 ){ //判定有
							block[bcnt].flg = 2;
							tflg = 0;
							score += 1;
							//効果音再生
							channel = Mix_PlayChannel(-1,bom,0);
							break;
						}
					}

					for(bcnt = 0;bcnt < BALLMAX;bcnt++){
						if( CheckShot(mbx,mby,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1 ){ //判定有
							ball[bcnt].flg = 2;
							tflg = 0;
							score += 2;
							//効果音再生
							channel = Mix_PlayChannel(-1,bom,0);
							break;
						}
					}

					tflg = 0;
				}

				//==================================================
				//ブロックに関する
				//==================================================
				binitcnt++;
				for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
					if(block[bcnt].flg == 0){ //未出現時
						//if( (binitcnt % 100-(int)score/2) == 0){ //出現間隔の調整
						if( binitcnt > 100-score/2 ){
							block[bcnt].x = blockwidth*(rand()%10);
							block[bcnt].y = -blockheight/2;
							block[bcnt].vy = 0;
							block[bcnt].gravity = 0;
							block[bcnt].flg = 1;
							binitcnt = 0;
						}
					}
					else if(block[bcnt].flg == 1){ //出現時
						//付加==============================================
						//体重分を重力へ
						block[bcnt].gravity += block[bcnt].weight;
						//重力付加を自機へ
						block[bcnt].vy += block[bcnt].gravity;
						
						//座標更新===========================================
						if(block[bcnt].vy > 0){
							for(v = 0;v < block[bcnt].vy;v++){

								//自機潰し判定有
								if( CheckObjectDown(x,y,width,height,block[bcnt].x,block[bcnt].y,blockwidth,blockheight) == 1){
									flg = 1;
									wy = height*3;
									wx = 0;
									block[bcnt].flg = 0;
									block[bcnt].x = -blockwidth*2;
									break;
								}

								if( CheckDownCollor(screen,block[bcnt].x,block[bcnt].y,blockwidth,blockheight) == 0){ //下の色判定無
									block[bcnt].y++;
								}
								else{
									block[bcnt].vy = 0;
									block[bcnt].gravity = 0;
									break;
								}

							}
						}
					}
					else if(block[bcnt].flg == 2){ //破壊されたブロック
						block[bcnt].wy+=blockheight; //画像の座標をずらしていく
						if(block[bcnt].wy == blockheight*9){ //完全に消えたら
							block[bcnt].flg = 0; //未出現にする
							block[bcnt].wy = 0;
							block[bcnt].x = -blockwidth;
						}

					} //block_flg END

				} //for END

				//==================================================
				//ボールに関する
				//==================================================
				for(bcnt = 0;bcnt < BALLMAX;bcnt++){
					if(ball[bcnt].flg == 0){ //未出現時
						if( binitcnt == 0 && rand()%(100+score) == 0 ){
							ball[bcnt].x = VIDEO_W/2 - ballwidth/2;
							ball[bcnt].y = -ballheight/2;
							ball[bcnt].vx = (rand()%10)-5;
							ball[bcnt].vy = 0;
							ball[bcnt].gravity = 0;
							ball[bcnt].flg = 1;
							binitcnt = 1;
						}
					}
					else if(ball[bcnt].flg == 1){ //出現時
						//付加==============================================
						//体重分を重力へ
						ball[bcnt].gravity += ball[bcnt].weight;
						//重力付加を自機へ
						ball[bcnt].vy += ball[bcnt].gravity;

						if( CheckUpCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1){ //上の色判定有
							ball[bcnt].vx += CheckUpMirror(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight);
						}

						if( CheckDownCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1){ //下の色判定有
							ball[bcnt].vx += CheckDownMirror(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight);	
							if( ball[bcnt].vx > 0){
								ball[bcnt].vx-=0.1;
								if(ball[bcnt].vx < 0) ball[bcnt].vx = 0;
							}
							else if( ball[bcnt].vx < 0){
								ball[bcnt].vx+=0.1;
								if(ball[bcnt].vx > 0) ball[bcnt].vx = 0;
							}
						}

						//座標更新===========================================
						if(ball[bcnt].vy > 0){
							for(v = 0;v < ball[bcnt].vy;v++){

								//自機潰し判定有
								if( CheckObjectDown(x,y,width,height,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1){
									flg = 1;
									wy = height*3;
									wx = 0;
									ball[bcnt].flg = 0;
									ball[bcnt].x = -ballwidth*2;
									break;
								}

								if( CheckDownCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 0){ //下の色判定無
									ball[bcnt].y++;
								}
								else{
									if(ball[bcnt].vy > 2) ball[bcnt].vy = -ball[bcnt].vy*0.3;
									else ball[bcnt].vy = 0;
									ball[bcnt].gravity = 0;
									break;
								}
							}
						}
						else if(ball[bcnt].vy < 0){
							for(v = 0;v > ball[bcnt].vy;v--){
								if( CheckUpCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 0){ //上の色判定無
									if( CheckObject(x,y,width,height,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1 && CheckDownCollor(screen,x,y,width,height) == 1){ //自機下の色判定有
										y--;
									}
									ball[bcnt].y--;
								}
								else{
									ball[bcnt].vy = 0;
									break;
								}
							}

						} //vy END

						if(ball[bcnt].vx > 0){
							for(v = 0;v < ball[bcnt].vx;v++){
								if( CheckRightCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 0){ //右の色判定無

									if( CheckObject(x,y,width,height,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1 &&
										CheckLeftCollor(screen,x,y,width,height) == 1){ //自機左の色判定有
											x++;
											//自機が押され、潰される処理
											if( CheckRightCollor(screen,x,y,width,height) == 1){
												flg = 1;
											}
									}

									ball[bcnt].x++;
								}
								else{
									if(ball[bcnt].vx > 0.1) ball[bcnt].vx = -ball[bcnt].vx*0.5;
									else ball[bcnt].vx = 0;
									break;
								}
							}
						}
						else if(ball[bcnt].vx < 0){
							for(v = 0;v > ball[bcnt].vx;v--){
								if( CheckLeftCollor(screen,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 0){ //左の色判定無

									if( CheckObject(x,y,width,height,ball[bcnt].x,ball[bcnt].y,ballwidth,ballheight) == 1 &&
										CheckRightCollor(screen,x,y,width,height) == 1){ //自機右の色判定有
											x--;
											//自機が押され、潰される処理
											if( CheckLeftCollor(screen,x,y,width,height) == 1){
												flg = 1;
											}
									}

									ball[bcnt].x--;
								}
								else{
									if(ball[bcnt].vx < -0.1)ball[bcnt].vx = -ball[bcnt].vx*0.5;
									else ball[bcnt].vx = 0;
									break;
								}
							}

						} //vx END

					}
					else if(ball[bcnt].flg == 2){ //破壊されたボール
						ball[bcnt].wy+=ballheight; //画像の座標をずらしていく
						if(ball[bcnt].wy == ballheight*9){ //完全に消えたら
							ball[bcnt].flg = 0; //未出現にする
							ball[bcnt].wy = 0;
							ball[bcnt].x = -ballwidth;
						}

					} //ball_flg END

				} //for END

			}
			else{ //死亡時
				//効果音再生
				channel = Mix_PlayChannel(-1,bom,0);
				flg++; //2~
				wy += height;
				//height -= 4;

				if(flg == 8){ //9
					gameflg = 2;
					Mix_HaltMusic(); //BGM停止
					Mix_PlayMusic(title,-1); //BGM再生
				}
			} //flg END

			//==================================================
			//描画処理
			//==================================================
			fmt = screen->format;						//サーフェスからパレット情報格納
			white = SDL_MapRGB(fmt, 0xff, 0xff, 0xff);	//R,G,B をパレット情報に合わせて格納
			//指定した色で塗りつぶす
			SDL_FillRect(screen, NULL, white);			//格納した色で塗りつぶす


			//描画==============================================
			//自機描画
			src.w = width; src.h = height;
			src.x = wx; src.y = wy;
			dsr.x = x; dsr.y = y;
			SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			//ブロック描画
			src.w = blockwidth; src.h = blockheight;
			for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
				src.x = block[bcnt].wx; src.y = block[bcnt].wy;
				dsr.x = block[bcnt].x; dsr.y = block[bcnt].y;
				SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//ボール描画
			src.w = ballwidth; src.h = ballheight;
			for(bcnt = 0;bcnt < BALLMAX;bcnt++){
				src.x = ball[bcnt].wx; src.y = ball[bcnt].wy;
				dsr.x = ball[bcnt].x; dsr.y = ball[bcnt].y;
				SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//天井
			src.w = 32; src.h = 32;
			for(i = 0;i < VIDEO_W/src.w;i++){
				src.x = 16; src.y = 0;
				dsr.x = i*src.w; dsr.y = 0;
				SDL_BlitSurface(image1,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//SCORE
			src.w = 50; src.h = 20;
			src.x = 0; src.y = 110;
			dsr.x = 0; dsr.y = 0;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			for(i = 0;i < 3;i++){
				switch(i){
					case 0: scorecnt = score/100; break;
					case 1: scorecnt = (score/10)%10; break;
					case 2: scorecnt = score%10; break;
				}
				src.w = 15; src.h = 20;
				src.x = src.w*scorecnt; src.y = 0;
				dsr.x = 60+src.w*i; dsr.y = 0;
				SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//HIGHSCORE
			src.w = 80; src.h = 20;
			src.x = 50; src.y = 110;
			dsr.x = VIDEO_W-140; dsr.y = 0;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			for(i = 0;i < 3;i++){
				switch(i){
					case 0: scorecnt = highscore/100; break;
					case 1: scorecnt = (highscore/10)%10; break;
					case 2: scorecnt = highscore%10; break;
				}
				src.w = 15; src.h = 20;
				src.x = src.w*scorecnt; src.y = 0;
				dsr.x = VIDEO_W-src.w*3+src.w*i; dsr.y = 0;
				SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

		}

		//====================================================================================================
		//リザルト画面
		//====================================================================================================
		else if(gameflg == 2){

			//すたーと カーソル
			if(VIDEO_W/2-50 < mx && VIDEO_W/2+50 > mx &&
				VIDEO_H/2 < my && VIDEO_H/2+30 > my){
					if(titleflg == 0){
						//効果音再生
						channel = Mix_PlayChannel(-1,cursor,0);
						titleflg = 1;
					}
			}else{
				titleflg = 0;
			}

			if(tflg == 1){
				//たいとる クリック
//				if( CheckShot(mbx,mby,VIDEO_W/2-50,VIDEO_H/2,100,30) == 1 ){
				if(VIDEO_W/2-50 < mbx && VIDEO_W/2+50 > mbx &&
					VIDEO_H/2 < mby && VIDEO_H/2+30 > mby){
						//効果音再生
						channel = Mix_PlayChannel(-1,select,0);
						//スコア更新チェック
						if(highscore < score) highscore = score;

						//各初期化
						/*
						//自機に関する
						MineInit(flg, jflg, wx, wy, width, height, x, y, vx, vy, weight, gravity);
						//弾に関する
						TamaInit(tflg, twx, twy, twidth, theight, tx, ty, tvx, tvy, tang);
						//ブロックに関する
						BlockInit(block, blockwidth, blockheight);
						//ボールに関する
						BallInit(block, ballwidth, ballheight);
						*/

						flg = 0;
						jflg = 0;
						wx = 0; wy = 0;
						width = 16; height = 32;
						x = VIDEO_W/2; y = VIDEO_H/2;
						vx = 0;	vy = 0;
						weight = 0.5; gravity = 0;

						tflg = 0;
						twx = 0, twy = 0, twidth = 0, theight = 0;
						tx = 0, ty = 0;
						tvx = 0, tvy = 0;
						tang = 0;

						blockwidth = 32, blockheight = 32;
						for(bcnt = 0;bcnt < BLOCKMAX;bcnt++){
							block[bcnt].wx = 64;			//画像座標x
							block[bcnt].wy = 0;			//画像座標y
							block[bcnt].x = -blockwidth;		//座標x
							block[bcnt].y = -blockheight;		//座標y
							block[bcnt].vx = 0;			//ベクトルx
							block[bcnt].vy = 0;			//ベクトルy
							block[bcnt].weight = 0.01;		//体重
							block[bcnt].gravity = 0;		//重力
							block[bcnt].flg = 0;			//状態
						}

						ballwidth = 32, ballheight = 32;
						for(bcnt = 0;bcnt < BALLMAX;bcnt++){
							ball[bcnt].wx = 96;			//画像座標x
							ball[bcnt].wy = 0;			//画像座標y
							ball[bcnt].x = -ballwidth;		//座標x
							ball[bcnt].y = -ballheight;		//座標y
							ball[bcnt].vx = 0;			//ベクトルx
							ball[bcnt].vy = 0;			//ベクトルy
							ball[bcnt].weight = 0.005;		//体重
							ball[bcnt].gravity = 0;			//重力
							ball[bcnt].flg = 0;			//状態
						}

						gameflg = 0; //タイトルへ
						//BGM再生
						Mix_PlayMusic(title,-1);
				}
			}

			//==================================================
			//描画処理
			//==================================================
			fmt = screen->format;						//サーフェスからパレット情報格納
			white = SDL_MapRGB(fmt, 0xff, 0xff, 0xff);	//R,G,B をパレット情報に合わせて格納
			//指定した色で塗りつぶす
			SDL_FillRect(screen, NULL, white);			//格納した色で塗りつぶす

			//スコア表示
			src.w = 50; src.h = 20;
			src.x = 0; src.y = 110;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = 30;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

			for(i = 0;i < 3;i++){
				switch(i){
					case 0: scorecnt = score/100; break;
					case 1: scorecnt = (score/10)%10; break;
					case 2: scorecnt = score%10; break;
				}
				src.w = 15; src.h = 20;
				src.x = src.w*scorecnt; src.y = 0;
				dsr.x = VIDEO_W/2-(src.w*3)/2+src.w*i; dsr.y = 60;
				SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報
			}

			//たいとる
			src.w = 100; src.h = 30;
			src.x = 100*titleflg; src.y = 80;
			dsr.x = VIDEO_W/2-src.w/2; dsr.y = VIDEO_H/2;
			SDL_BlitSurface(image2,&src,screen,&dsr); //転送元メモリ,転送元情報,転送先メモリ,転送先情報

		} //gameflg END

		//FPS調整用
		e_time=SDL_GetTicks();
		if(s_flg==1){
			fmt=screen->format; 			// サーフェスからパレット情報格納
			white=SDL_MapRGB(fmt,0xff, 0xff, 0xff);	// R,G,B　をパレット情報に合わせて、格納
			SDL_FillRect(screen, NULL,white);	// 格納した色で塗りつぶす
			h_time=(float)(e_time-s_time)/1000;
			printf("%3.2f\n",h_time);
			s_flg=0;
		}

		//デバッグ用
		printf("%d\n",jflg);
//		printf("x,y(%4.1f,%4.1f) vx,vy(%4.1f,%4.1f) gravity(%4.1f) score(%d)\n",x,y,vx,vy,gravity,score);
//		static int color;
//		color=getpixel(screen,mbx,mby);
//		printf("color %d\n",color);

		//ダブルバッファバージョン 返値０・・・成功
		SDL_Flip(screen);
		loopWait();
		calcFPS();

	} //while END

	//====================================================================================================
	//後始末
	//====================================================================================================

	SDL_FreeSurface(screen);
	SDL_FreeSurface(image1);
	SDL_FreeSurface(image2);

	Mix_HaltMusic();			//BGM停止
	Mix_HaltChannel(channel);		//効果音停止
	Mix_FreeMusic(title);			//BGM用のメモリ解法
	Mix_FreeMusic(game);			//BGM用のメモリ解法
	Mix_FreeChunk(cursor);			//効果音用のメモリ解法
	Mix_FreeChunk(select);			//効果音用のメモリ解法
	Mix_FreeChunk(bom);			//効果音用のメモリ解法
	Mix_CloseAudio();

	printf("OFF\n");
	SDL_Quit();
	return 0;
} //main END

//====================================================================================================
